/*
termex-en.js
  - termextract-brillstaggerjs sample script
*/
const termextract = require("./termextract-brillstaggerjs.cjs");
const natural = require("natural");

const fs = require("fs");
const { exit } = require("process");
if (process.argv[2] == undefined) {
  console.log(
    `Not enough command line arguments!
  Usage:
   node teremex-eng.js inputfile.txt outputfile.txt`
  );
  exit();
}
const inputfile = process.argv[2];
const outputfile = process.argv[3] || "./outputfile.txt";
let strings = fs.readFileSync(inputfile, 'utf8');

const language = "EN"
const defaultCategory = 'N';
const defaultCategoryCapitalized = 'NNP';
const lexicon = new natural.Lexicon(language, defaultCategory, defaultCategoryCapitalized);
const ruleSet = new natural.RuleSet('EN');
const tagger = new natural.BrillPOSTagger(lexicon, ruleSet);

fs.open(outputfile, 'w', (err, fd) => {
  strings = strings.replace(/([\.\,\?\!])/g, " $1");
  strings = strings.split("\n").join(' \n ');
  strings = strings.split("\r").join(' \r ');
  let sentence = strings.split(/\s+/);
  let tokenized_word = tagger.tag(sentence);
  let cmp_noun_list = termextract.cmp_noun_list(tokenized_word);
  let frequency = termextract.list2key_value(cmp_noun_list);
  let score_lr = termextract.score_lr(frequency, termextract.IGNORE_WORDS, 1, 1);
  let term_importance = termextract.term_importance(frequency, score_lr);
  let score_lt_list = termextract.sort_by_importance(term_importance);
  for (const data of score_lt_list) {
      fs.writeFileSync(fd,`${data.cmp_noun}\t${data.importance}\n`);
  }
});
