/*
termextract-brillstaggerjs
POStagger Brills Taggerをベースとした英語の専門用語自動抽出システム

Copyright
「専門用語自動抽出システム」は 東京大学情報基盤センター図書館電子化部門中川裕志教授および 横浜国立大学環境情報研究院森辰則助教授が共同で開発したものです。
termextract-brillstaggerjsは主として中川教授、 東京大学前田朗、 東京大学小島浩之講師の３者で検討を重ね、この「専門用語自動抽出システム」を改良したTermExtractをベースに、日本語の専門用語自動抽出の基本機能をnode.js(JavaScript)に移植したものである。
*/
const version = '0.01b';
exports.version = version;
const core = require("./termextract-core.cjs");
const IGNORE_WORDS = new Set(["of", "Of", "OF"]);  // 重要度計算外とする語
exports.IGNORE_WORDS = IGNORE_WORDS;
exports.cmp_noun_list = cmp_noun_list;
exports.list2key_value = core.list2key_value;
exports.score_lr = core.score_lr;
exports.term_importance = core.term_importance;
exports.sort_by_importance = core.sort_by_importance;
exports.modify_agglutinative_lang = core.Zmodify_agglutinative_lang;
let cmp_nouns = [];

function cmp_noun_list(data) {
  cmp_nouns = [];
  let status = 0
  /*
  status = 1   前が名詞(NN, NNS, NNP)
           2   前が形容詞(JJ)
           3   前が所有格語尾(POS)
           4   前がof
           5   前が基数(CD)
           6   前が過去分詞の動詞(VBN)
           7   前が外来語(FW)
  */
  let rest = 0  // 名詞以外の語が何語連続したかカウント
  let seg = []  // 複合語のリスト
  let taggedWords = data["taggedWords"];
  for (const morph of taggedWords) {
    let term = morph["token"];
    let tag = morph["tag"];
    // 数値や区切り記号の場合
    if (term.match(/[\s\+\%\&\$\*\#\^\|\[\]\-]$/) || term.match(/\d+$/)){
         _increase(cmp_nouns, seg, rest);
         seg = [];
         continue;
    }
    // 名詞の場合
    if (tag.match(/NN[PS]?$/) || tag == "NNPS"){
      // 複数形を単数形に置き換える。
      if (tag == "NNS"){
        term = stemming(term);
      }
      // 固有名詞以外は先頭の大文字を小文字に。
      if (tag.match(/NNS?$/) && term.match(/^[A-Z][a-z]/)){
        term = term.substring(0, 1).toLowerCase() + term.substring(1);
      }
      status = 1
      seg.push(term)
      rest = 0
    }
    // 形容詞(JJ)の場合
    else if (tag == "JJ"){
      // 前の語が"なし","形容詞","所有格語尾","基数"の場合は連結する
      if (status == 0 || status == 2 || status == 3 || status == 5){
        seg.push(term);
        rest += 1;
      }
      else{
        _increase(cmp_nouns, seg, rest);
        seg = [term];
        rest += 1;
      }
      status = 2;
    }
    // 所有格語尾(POS)の場合
    else if (tag == "POS"){
      // 前の語が名詞の場合は連結する
      if (status == 1){
        status = 3;
        seg.push(term);
        rest += 1;
      }
      else{
        _increase(cmp_nouns, seg, rest);
        seg = [];
      }
    }
    // of の場合
    else if (term == "of" && tag == "IN"){
      // 前の語が名詞の場合は連結する
      if (status == 1){
        status = 4;
        seg.push(term);
        rest += 1;
      }
      else{
        _increase(cmp_nouns, seg, rest);
        seg = [];
        status = 0;
      }
    }
    // 基数(CD)の場合は、語の先頭のみ許可
    else if (tag == "CD"){
      _increase(cmp_nouns, seg, rest);
      seg = [term];
      status = 5;
    }
    // 過去分詞の動詞は語の先頭のみ許可
    else if (tag == "VBN"){
      _increase(cmp_nouns, seg, rest);
      seg = [term];
      status = 6;
      rest += 1;
    }
    // 外来語(FW)の場合は単語として処理
    else if (tag == "FW"){
      _increase(cmp_nouns, seg, rest);
      status = 7;
      seg = [term];
    }
    // 指定した品詞以外の場合は、そこで複合語の区切りとする
    else{
      _increase(cmp_nouns, seg, rest);
      seg = [];
      status = 0;
    }
  }
  // 改行があった場合はそこで複合語の区切りとする
  _increase(cmp_nouns, seg, rest);
  seg = [];
  status = 0;
  return cmp_nouns;
}

function stemming(noun){
  // 複数形を単数形に変えるだけのstemmer
  if (noun.match(/ies$/) && !(noun.match(/[ae]ies$/))){
     noun = noun.replace(/ies$/, "y");
  }
  else if (noun.match(/es$/) && !(noun.match(/[aeo]es$/))){
    noun = noun.replace(/es$/, "e");
  }
  else if (noun.match(/s$/) && !(noun.match(/[us]s/))){
    noun = noun.replace(/s$/, "");
  }
  return noun;
}

function _increase(cmp_nouns, seg, rest){
  let allwords = ""
  // 複合語の末尾は名詞とし、それ以外は切り捨てる
  let pos = seg.length - rest;
  if (pos > 0){
    seg = seg.slice(0, pos);
  }
  rest = 0;
  if (seg.length > 0){
    for(let word of seg) {
      word = word.replace(/^\s+/, "");
      if (allwords == ""){
        allwords = word;
      }
      else{
        allwords += ' ' + word;
      }
    }
    // ' で区切られた語は接続する
    allwords.replace(" ' ", "'")
    // 末尾の . と , は削除する
    allwords = allwords.replace(/,$/, "");
    allwords = allwords.replace(/\.$/, "");
    cmp_nouns.push(allwords);
  }
}
