/*
termextract-kuromojijs
日本語形態素解析システム kuromoji.jsをベースとした日本語の専門用語自動抽出システム

Copyright
「専門用語自動抽出システム」は 東京大学情報基盤センター図書館電子化部門中川裕志教授および 横浜国立大学環境情報研究院森辰則助教授が共同で開発したものです。
termextract-kuromojijsは主として中川教授、 東京大学前田朗、 東京大学小島浩之講師の３者で検討を重ね、この「専門用語自動抽出システム」を改良したTermExtractをベースに、日本語の専門用語自動抽出の基本機能をnode.js(JavaScript)に移植したものである。
*/
const version = '0.01b';
exports.version = version;
const IGNORE_WORDS = new Set(["的"]);  // 重要度計算外とする語
const core = require("./termextract-core.cjs");
exports.IGNORE_WORDS = IGNORE_WORDS;
exports.cmp_noun_list = cmp_noun_list;
exports.list2key_value = core.list2key_value;
exports.score_lr = core.score_lr;
exports.term_importance = core.term_importance;
exports.sort_by_importance = core.sort_by_importance;
exports.modify_agglutinative_lang = core.modify_agglutinative_lang;
let cmp_nouns = [];
let terms = [];  // 複合語リスト作成用の作業用配列

function cmp_noun_list(data) {
  // 和布蕪の形態素解析結果を受け取り、複合語（空白区切りの単名詞）のリストを返す
  let must = 0;  // 次の語が名詞でなければならない場合は真
  const re = "[!\"#$%&'\(\)*+,-./{\|}:;<>\[\]\?!]$";
  cmp_nouns = [];
  // 単名詞の連結処理
  for (const item of data) {
    let noun = item.surface_form;
    let pos = item.pos;
    let cl_1 = item.pos_detail_1;
    let cl_2 = item.pos_detail_2;
    if (pos == "") {
      if (!must) {
        increase(cmp_nouns, terms);
      }
      must = 0;
      continue;
    }
    if (pos === "名詞" && cl_1 === "一般" ||
        pos === "名詞" && cl_1 === "接尾" && cl_2 === "一般" ||
        pos === "名詞" && cl_1 === "接尾" && cl_2 === "サ変接続" ||
        pos === "名詞" && cl_1 === "固有名詞" ||
        pos === "記号" && cl_1 === "アルファベット" ||
        pos === "名詞" && cl_1 === "サ変接続" && !noun.match(re)
      ) {
      terms.push(noun);
      must = 0;
    }
    else if (pos === "名詞" && cl_1 === "形容動詞語幹" ||
      pos == "名詞" && cl_1 === "ナイ形容詞語幹") {
      terms.push(noun);
      must = 1;
    }
    else if (pos === "名詞" && cl_1 === "接尾" && cl_2 == "形容動詞語幹") {
      terms.push(noun);
      must = 1;
    }
    else if (pos === "動詞") {
      terms = [];
    }
    else {
      if (!must) {
        increase(cmp_nouns, terms); 
      }
      must = 0;
      terms = [];     
    }
  }
  if (!must) {
    increase(cmp_nouns, terms);  
  }
  return cmp_nouns;
}

function increase(cmp_nouns, terms) {
  const SETSUBI = new Set(["など", "ら", "上", "内", "型", "間", "中", "毎"]);
  const re = "\s+$";
  //  専門用語リストへ、整形して追加するサブルーチン
  if (terms.length > 1) {
    if (terms[0] === "本") {
      terms.shift();
    }
  }
  if (terms.length > 1) {
    // 語尾の余分な語の削除
    let end = terms.slice(-1)[0];
    if (SETSUBI.has(end) || end.match(re)) {
      terms.pop();
    }
  }
  if (terms.length > 0) {
    cmp_noun = terms.join(" ");
    cmp_nouns.push(cmp_noun);
    terms = [];
  }
}
